--[[
Plugin for showing a warning text in the middle of the screen (or desired place)
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("WarningFrame")
if not plugin then return end

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Warnings")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local FA = LibStub("LibFrameAnchorRegistry-1.0")
local Utils = RW.Utils
local Party
local db
local UIF

----------------------------------
-- Colors
plugin.Colors = {
	{
		name = "Posetive",
		description = "Posetive messages",
		color = {0, 1, 0},
	},
	{
		name = "Negative",
		description = "Negative messages",
		color = {1, 0.15, 0.15},
	},
	{
		name = "Information",
		description = "Informative messages",
		color = {1, 0.95, 0.64},
	},
	{
		name = "Personal",
		description = "Personal messages only shown to you",
		color = {0.33, 0.7, 1},
	}
}

----------------------------------
-- Defaults

local defaults = {
	profile = {
		Width = 180,
		DefaultColor = {0, 0.5, 1},
		DefaultSize = 32,
		DefaultSound = "RW: Tata",
		PlaySound = true,
		UseFlash = true,
		PluginEnabled = true,
		x = 0,
		y = 0,
		AnchorFrame = nil,
		Point = "CENTER",
		RelPoint = "CENTER",
		
		FontAlign = "LEFT",
		
		FontSettings = {
			Font = "accid",
			Size = 40,
			Flags = "OUTLINE",
			Color = {0.47, 0.75, 1, 1},
		},
		
		BGTexture = "Solid",
		BGColor = {0, 0, 0, 0.5},
		Border = "IshBorder",
		BorderColor = {0.15, 0.24, 0.31, 1},
		BorderSize = 12,
		BarTexture = "HalD",
		BarColor = {0.22, 0.34, 0.46, 1},
		Insets = 1,
		BorderMargin = 5,
		TextHeight = 1,
		OverSize = false,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	-- DB Upgrade
	for name, profile in pairs(self.db.profiles) do
		local db = profile
		db.FontSettings = db.FontSettings or {}
		if db.Font then	db.FontSettings.Font = db.Font end
		if db.FontSize then db.FontSettings.Size = db.FontSize end
		if db.FontOutline then db.FontSettings.Flags = db.FontOutline end
		
		db.Font = nil
		db.FontSize = nil
		db.FontOutline = nil
		db.FontColor = nil	
	end	
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
	
	self:RegisterCallback("ConfigMode")
	
	self:RegisterCallback("WarningShow")
	
	Party = RW.Party
end

function plugin:PluginDisable()
	self:UnregisterAllCallbacks()
end

-----------------------------------
-- Anchor

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)

	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	db.AnchorFrame = nil
	db.Point = "CENTER"
	db.RelPoint = "CENTER"
	
	RW.Callbacks:Fire("RefreshOptions")
end

local function getAnchor()
	if plugin.anchor then return plugin.anchor end
	local frame = Utils:CreateAnchor(plugin:GetName(), UIParent, "Warning frame")
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	
	plugin.anchor = frame		
	
	plugin:Reset()
	
	return frame
end

local function getWarningFrame()
	if plugin.warningFrame then return plugin.warningFrame end
	local db = plugin.db.profile
	local anchor = getAnchor()
	
	local frame = CreateFrame("Frame", "RW_WarningFrame")
	frame:SetPoint("BOTTOM", anchor, "TOP")
	frame:SetWidth(db.Width)
	frame:SetHeight(1)
	
	local text = frame:CreateFontString(frame:GetName().."String", "ARTWORK")
	text:SetPoint("BOTTOM", frame, "TOP")
	
	local icon = frame:CreateTexture(frame:GetName().."Icon", "OVERLAY")
	icon:SetPoint("RIGHT", text, "LEFT", -10, 0)
	icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	
	local iconback = frame:CreateTexture(nil, "ARTOWRK")
	iconback:SetTexture(0, 0, 0, 1)
	iconback:SetPoint("TOPLEFT", icon, -3, 3)
	iconback:SetPoint("BOTTOMRIGHT", icon, 3, -3)
	
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.timer = self.timer - elapsed
		if self.timer >= 3 and self.timer <= 4 then
			LowHealthFrame:SetAlpha(self.timer - 3)
		elseif self.timer <= 0.1 then
			frame:Hide()
		elseif self.timer <= 2 then
			frame:SetAlpha(self.timer/2)
		end
	end)
	
	frame.text = text
	frame.icon = icon
	plugin.warningFrame = frame
	plugin:ApplySettings()
	return frame
end

-----------------------------------
-- Warning functions

-- Reset position
function plugin:Reset()
	db = self.db.profile
	if self.anchor then
		local frame = self.anchor
		local scale = frame:GetEffectiveScale() / UIParent:GetEffectiveScale()
		
		local anchor = FA:GetAnchor(db.AnchorFrame or "BlizzUIParent")
		if not anchor then anchor = UIParent end
		
		frame:ClearAllPoints()			
		frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / scale, db.y / scale)
		frame:SetWidth(db.Width)
		
		plugin:ApplySettings()
	end
end

-- Apply the visual settings
function plugin:ApplySettings()
	if not plugin.warningFrame then return end
	local frame = getWarningFrame()
		
	frame.text:SetFont(LSM:Fetch("font", db.FontSettings.Font), db.FontSettings.Size, db.FontSettings.Flags )	
	frame.text:SetText(frame.text:GetText() or "T")
	frame.text:SetTextColor(unpack(db.DefaultColor))
	if db.OverSize then
		frame.text:SetTextHeight(db.TextHeight)
	end
	frame.icon:SetWidth(frame.text:GetStringHeight())
	frame.icon:SetHeight(frame.text:GetStringHeight())	
end

--- Show a warning message
-- @param text The text to shows 
-- @param icon Optional icon to the left of the text
-- @param color Optional color for the warning, else it will be the default color
function plugin:WarningShow(event, text, icon, color, sound, flash)
	local frame = getWarningFrame()
	local c = color or self.db.profile.DefaultColor
	frame.text:SetText(text)
	frame.text:SetTextColor(unpack(c))
	frame.icon:SetTexture(icon)
	frame:Show()
	frame:SetAlpha(1)
	frame.timer = 5
	
	if db.PlaySound then
		local soundfile = LSM:Fetch("sound", sound)
		if soundfile == "Interface\\Quiet.ogg" then soundfile = sound or LSM:Fetch("sound", db.DefaultSound) or soundfile end
		PlaySoundFile(soundfile)
	end
	if db.UseFlash and flash then
		RW.Callbacks:Fire("FlashShow", color)
	end
end

function plugin:ConfigMode(event, start)
	if start then
		getAnchor():Show()
		getWarningFrame():Show()
		getWarningFrame():SetAlpha(1)
		getWarningFrame().timer = 11111111
		getWarningFrame().text:SetText("Warning Frame")
	else
		getAnchor():Hide()
		getWarningFrame():Hide()
		getWarningFrame().timer = 0
	end
end

function plugin:GetOptions()
	return {{text = LB.PLUGINS, value = "Plugins"}, {text = LB.ALERT_PLUGINS, value = "AlertPlugins"}, {text = L.NAME, value = "WarningFrame"}}, self.GetGUI, [[Interface\Icons\INV_Misc_ClothScrap_02]]
end

local function changeCallback()
	plugin:Reset()
end

do
	local AceGUI = LibStub("AceGUI-3.0")
	local TabSelected
	local LastTab

	function plugin:GetGUI()
		local self = self or plugin		
		UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
		
		local group, label, slider, check, igroup, button, color, select
		
		local group = AceGUI:Create("SimpleGroup")	
		group:SetFullWidth(true)
		group:SetLayout("Flow")
		
		label = UIF:MainTitleLabel(L.NAME)
		group:AddChild(label)
		
		igroup = UIF:InlineGroup("")
		group:AddChild(igroup)	
		
		label = UIF:NormalText(L.DESC)
		label:SetImage([[Interface\Icons\INV_Misc_ClothScrap_02]])
		igroup:AddChild(label)
		
		group:AddChild(UIF:CheckBox(LB.ENABLED, db, "PluginEnabled", function() 
			if not db.PluginEnabled then
				self:Disable()
			else
				self:Enable()
			end
		end, nil, 0.3))
		button = UIF:Button(LB.TEST, function() 
			local spellName, rank, icon = Utils:GetRandomSpell()
			RW.Callbacks:Fire("WarningShow", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), icon, nil, db.DefaultSound, true) 
		end)
		group:AddChild(button)
		
		local tgroup = UIF:TabGroup()
		group:AddChild(tgroup)
		tgroup:SetTabs({
			{text = "Misc", value = "Misc"},
			{text = LB.POSITION, value = "Position"},
			{text = LB.FONT, value = "Font"},
		})
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastTab or "Misc")	
		
		button = UIF:Button(LB.RESET, function() 
			plugin.db:ResetProfile() 
		end)
		button:SetUserData("confirm", true)
		button:SetUserData("TriggerGlobalUpdate", true)
		group:AddChild(button)
		
		Utils:DoLayout(group)
		
		return group
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastTab = tab
		
		if tab == "Misc" then
			self:AddChild(UIF:CheckBox(L.FLASH, db, "UseFlash", changeCallback, nil, 0.5))		
			self:AddChild(UIF:CheckBox(LB.PLAY_SOUND, db, "PlaySound", changeCallback, nil, 0.5))
			self:AddChild(UIF:ColorSelect(LB.DEFAULT_COLOR, db, "DefaultColor", changeCallback))
			self:AddChild(UIF:NewLine())
			self:AddChild(UIF:LSMDropdown("sound", LB.DEFAULT_SOUND , db, "DefaultSound"))			
		
		elseif tab == "Position" then
			getAnchor()
			local t = AceGUI:Create("FrameAnchorDropdown")
			t:SetRelativeWidth(0.5)			
			t:SetList(FA:GetAnchors(plugin.anchor))
			t:SetLabel("Anchor")
			t:SetValue(db.AnchorFrame)
			t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
				db.AnchorFrame = cat
				changeCallback()
			end)	
			self:AddChild(t)
			self:AddChild(UIF:NewLine())
			
			self:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, changeCallback, 0.5, 75))
			self:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, changeCallback, 0.5, 75))
			
			slider = UIF:Slider("X", db, "x", 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
				1,
				changeCallback)
			self:AddChild(slider)
			
			slider = UIF:Slider("Y", db, "y", 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				1,
				changeCallback)
			self:AddChild(slider)
			
		elseif tab == "Font" then
			self:AddChild(UIF:FontSettings(LB.FONT_MAIN, db.FontSettings, false, changeCallback, 1))
			self:AddChild(UIF:NewLine())
			self:AddChild(UIF:CheckBox("Oversize", db, "OverSize", changeCallback, "Using this will make it posible to enlarge the font even more, but may distort it. May need reload after it is disable to restore.", 0.5))
			self:AddChild(UIF:Slider("Oversize Amount", db, "TextHeight", 0.1, 200, 0.1, changeCallback, 0.5))
		end
		
		Utils:DoLayout(self)
	end
end